import type { ThemeData } from './themeTypes.js';
export type DatePrecision = 'year' | 'half' | 'quarter' | 'month' | 'week' | 'day' | 'day-minutes' | 'day-seconds';
export type NumberColumnFormatterConfig = {
    'number-format'?: string;
    'number-divisor'?: number | string;
    'number-append'?: string;
    'number-prepend'?: string;
};
export type DateColumnFormatterConfig = {
    normalizeDatesToEn?: boolean;
};
export type ColumnTypeExpanded = {
    format(): unknown;
    precision(): DatePrecision;
};
export type Column = {
    range(): [any, any];
    name: () => string;
    value: (index: number) => string | number | boolean | null;
    values: () => (string | number | boolean | null)[];
    title: () => string;
    type(): 'text' | 'number' | 'date';
    type(expand: true): ColumnTypeExpanded;
    raw(): string;
};
export type Metadata = {
    axes?: any;
    describe?: any;
    visualize?: any;
    annotate?: any;
    custom?: {
        webToPrint?: {
            mode?: string;
        };
    } & Record<`exif-${string}`, string>;
    data?: {
        'column-format'?: Record<string, Record<string, unknown>>;
        'column-order'?: number[];
        transpose?: boolean;
        'upload-method'?: 'external-data' | 'google-spreadsheet' | 'copy';
        'google-spreadsheet-src'?: string;
        'external-data'?: string;
        'external-metadata'?: string;
        changes?: {
            row: number;
            column: number;
            value: string | number;
            time?: number;
            previous?: string | number;
        }[];
    };
    publish?: {
        'embed-height'?: number;
        'embed-width'?: number;
    };
};
export type Overlay = {
    color: number | string;
    opacity: number | `${number}`;
};
export type TextDirection = 'rtl' | 'ltr';
export type Row = Record<string, string | number | boolean | null | Date>;
export type Dataset = {
    columns: () => Column[];
    column: (name: string | number) => Column;
    hasColumn: (name: string | number) => boolean;
    list: () => Row[];
    toCSV: () => string;
    numRows: () => number;
    numColumns: () => number;
};
export type VisAxesColumns = Record<string, Column[]>;
export type VisAxesNames = Record<string, string[]>;
type AxesGetter = {
    (asColumns?: false): VisAxesNames;
    (asColumns: true): VisAxesColumns;
};
type SettingsGetter<TRoot> = {
    (): TRoot;
    <TDefault>(key: string, default_?: TDefault): TDefault;
};
export type RenderFlags = {
    allowEditing?: boolean;
    dark?: boolean | 'auto';
    dev?: boolean;
    fitchart?: boolean;
    logo?: string;
    logoId?: string;
    map2svg?: boolean;
    plain?: boolean;
    previewId?: string;
    search?: string;
    static?: boolean;
    svgonly?: boolean;
    theme?: string;
    transparent?: boolean;
};
export type ChartLibraries = {
    chroma: typeof import('chroma-js');
    numeral: typeof import('numeral');
    dayjs: typeof import('dayjs');
};
export type DwChart = {
    get: SettingsGetter<Chart>;
    set: (key: string, value: unknown) => void;
    load: (data: string) => void;
    flags: () => RenderFlags;
    libraries: () => ChartLibraries;
    emotion: typeof import('@emotion/css');
};
export type Chart = {
    id: string;
    publicId: string;
    language: string;
    theme: string;
    type: string;
    title: string;
    lastEditStep: number;
    publicUrl?: string;
    publicVersion?: number;
    metadata: Metadata;
    thumbnails?: {
        plain: string;
    };
    thumbnailHash: string;
    folderId?: number | null;
    organizationId: string | null;
};
export type Visualization = {
    dataset: Dataset;
    get: SettingsGetter<object>;
    theme: () => ThemeData;
    chart: () => DwChart;
    darkMode: (darkMode: boolean) => boolean;
    axes: AxesGetter;
    libraries: () => ChartLibraries;
    addFilterUI: (args: {
        column: Column;
        type: 'auto' | 'select' | 'buttons' | 'timescale';
    }) => void;
    renderingComplete: () => void;
    size: () => [number, number];
    colorMap: () => (color: string) => string;
    colorMode: () => string;
    /**
     * Use this to wait until a visualization has finished rendering
     *
     * @returns {Promise} resolves when the rendering is completed
     */
    rendered: () => Promise<void>;
    __rendered: boolean;
    __lastRow: number;
    textDirection: TextDirection;
};
export {};
