"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadStylesheet = exports.loadScript = exports.deleteJSON = exports.patchJSON = exports.putJSON = exports.postJSON = exports.getJSON = exports.fetchJSON = void 0;
const getValueOrDefault_js_1 = require("./getValueOrDefault.cjs");
/**
 * Download and parse a remote JSON document. Use {@link httpReq} instead
 *
 * @deprecated
 *
 * @param {string} url
 * @param {string} method - HTTP method, either GET, POST or PUT
 * @param {string|undefined} credentials - set to "include" if cookies should be passed along CORS requests
 * @param {string} body
 * @param {function} callback
 *
 * @returns {Promise}
 *
 * @example
 * import { fetchJSON } from '@datawrapper/shared/fetch';
 * fetchJSON('http://api.example.org', 'GET', 'include');
 */
function fetchJSON(url, method, credentials, body, callback) {
    const opts = {
        method,
        body,
        mode: 'cors',
        credentials
    };
    return window
        .fetch(url, opts)
        .then(res => {
        if (!res.ok)
            throw new Error(res.statusText);
        return res.text();
    })
        .then(text => {
        try {
            return JSON.parse(text);
        }
        catch (Error) {
            // could not parse json, so just return text
            console.warn('malformed json input', text);
            return text;
        }
    })
        .then(res => {
        if (callback)
            callback(res);
        return res;
    })
        .catch(err => {
        if (callback) {
            console.error(err);
        }
        else {
            throw err;
        }
    });
}
exports.fetchJSON = fetchJSON;
function getJSON(url, inputCredentials, inputCallback) {
    let credentials;
    let callback;
    if (arguments.length === 2 && typeof inputCredentials === 'function') {
        // swap callback and assume default credentials
        callback = inputCredentials;
        credentials = 'include';
    }
    else if (arguments.length === 1) {
        callback = undefined;
        credentials = 'include';
    }
    else {
        callback = inputCallback;
        credentials = inputCredentials;
    }
    return fetchJSON(url, 'GET', credentials, null, callback);
}
exports.getJSON = getJSON;
/**
 * Download and parse a remote JSON endpoint via POST. credentials
 * are included automatically.
 * Use {@link httpReq} or {@link httpReq.post} instead.
 *
 * @deprecated
 *
 * @param {string} url
 * @param {string} body
 * @param {function} callback
 *
 * @returns {Promise}
 * @example
 * import { postJSON } from '@datawrapper/shared/fetch';
 *
 * postJSON('http://api.example.org', JSON.stringify({
 *    query: 'foo',
 *    page: 12
 * }));
 */
function postJSON(url, body, callback) {
    return fetchJSON(url, 'POST', 'include', body, callback);
}
exports.postJSON = postJSON;
/**
 * Download and parse a remote JSON endpoint via PUT. credentials
 * are included automatically
 * Use {@link httpReq} or {@link httpReq.put} instead.
 *
 * @deprecated
 *
 * @param {string} url
 * @param {string} body
 * @param {function} callback
 *
 * @returns {Promise}
 * @example
 * import { putJSON } from '@datawrapper/shared/fetch';
 *
 * putJSON('http://api.example.org', JSON.stringify({
 *    query: 'foo',
 *    page: 12
 * }));
 */
function putJSON(url, body, callback) {
    return fetchJSON(url, 'PUT', 'include', body, callback);
}
exports.putJSON = putJSON;
/**
 * @deprecated use httpReq or httpReq.patch instead
 *
 * Download and parse a remote JSON endpoint via PATCH. credentials
 * are included automatically
 *
 * @param {string} url
 * @param {string} body
 * @param {function} callback
 *
 * @returns {Promise}
 * @example
 * import { patchJSON } from '@datawrapper/shared/fetch';
 *
 * patchJSON('http://api.example.org', JSON.stringify({
 *    query: 'foo',
 *    page: 12
 * }));
 */
function patchJSON(url, body, callback) {
    return fetchJSON(url, 'PATCH', 'include', body, callback);
}
exports.patchJSON = patchJSON;
/**
 * Download and parse a remote JSON endpoint via DELETE. credentials
 * are included automatically
 * Use {@link httpReq} or {@link httpReq.delete} instead.
 *
 * @deprecated
 *
 * @param {string} url
 * @param {function} callback
 *
 * @returns {Promise}
 *
 * @example
 * import { deleteJSON } from '@datawrapper/shared/fetch';
 *
 * deleteJSON('http://api.example.org/chart/123').then(() => {
 *     console.log('deleted!')
 * });
 */
function deleteJSON(url, callback) {
    return fetchJSON(url, 'DELETE', 'include', null, callback);
}
exports.deleteJSON = deleteJSON;
/**
 * injects a `<script>` element to the page to load a new JS script
 *
 * @param {string} src
 * @param {function} callback
 *
 * @example
 * import { loadScript } from '@datawrapper/shared/fetch';
 *
 * loadScript('/static/js/library.js', () => {
 *     console.log('library is loaded');
 * })
 */
function loadScript(src, callback = null) {
    return new Promise((resolve, reject) => {
        const script = document.createElement('script');
        script.src = src;
        script.charset = 'utf-8';
        script.onload = () => {
            if (callback)
                callback();
            resolve();
        };
        script.onerror = reject;
        document.body.appendChild(script);
    });
}
exports.loadScript = loadScript;
/**
 * injects a `<link>` element to the page to load a new stylesheet
 *
 * @param {string|opts} src
 * @param {function} callback
 *
 * @example
 * import { loadStylesheet } from '@datawrapper/shared/fetch';
 *
 * loadStylesheet('/static/css/library.css', () => {
 *     console.log('library styles are loaded');
 * })
 */
function loadStylesheet(_opts, callback = null) {
    if (typeof _opts === 'string') {
        _opts = {
            src: _opts
        };
    }
    const opts = _opts;
    const targetElement = (0, getValueOrDefault_js_1.getValueOrDefault)(opts.parentElement, element => typeof element.appendChild === 'function', () => document.head);
    return new Promise((resolve, reject) => {
        const link = document.createElement('link');
        link.rel = 'stylesheet';
        link.href = opts.src;
        link.charset = 'utf-8';
        link.onload = () => {
            if (callback)
                callback();
            resolve();
        };
        link.onerror = reject;
        targetElement.appendChild(link);
    });
}
exports.loadStylesheet = loadStylesheet;
