"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tailLength_js_1 = __importDefault(require("./tailLength.cjs"));
const round_js_1 = __importDefault(require("./round.cjs"));
const underscore_1 = require("underscore");
/**
 * computes the significant dimension for a list of numbers
 * That's the number of decimals to which we can round the numbers
 * without loosing information
 *
 * @exports significantDimension
 * @kind function
 *
 * @example
 * import {significantDimension} from '@datawrapper/shared/significantDimension';
 * significantDimension([0,10,20,30]); // -1
 *
 * @param {number[]} values - list of input numbers
 * @param {number} tolerance - percent of input values that we allow to "collide"
 * @returns {number} - number of significant dimensions (= the number of decimals)
 */
function significantDimension(values, tolerance = 0.1) {
    let result = [];
    let decimals = 0;
    const uniqValues = (0, underscore_1.uniq)(values.filter(underscore_1.isFinite));
    const totalUniq = uniqValues.length;
    let check;
    let diff;
    const accepted = Math.floor(totalUniq * (1 - tolerance));
    if (uniqValues.length < 3) {
        // special case if there are only 2 unique values
        return Math.round(uniqValues.reduce(function (acc, cur) {
            if (!cur)
                return acc;
            const exp = Math.log(Math.abs(cur)) / Math.LN10;
            if (exp < 8 && exp > -3) {
                // use tail length for normal numbers
                return acc + Math.min(3, (0, tailLength_js_1.default)(uniqValues[0]));
            }
            else {
                return acc + (exp > 0 ? (exp - 1) * -1 : exp * -1);
            }
        }, 0) / uniqValues.length);
    }
    if ((0, underscore_1.uniq)(uniqValues.map(currentRound)).length > accepted) {
        // we seem to have enough precision, but maybe it's too much?
        check = function () {
            return (0, underscore_1.uniq)(result).length === totalUniq;
        };
        diff = -1;
    }
    else {
        // if we end up here it means we're loosing too much information
        // due to rounding, we need to increase precision
        check = function () {
            return (0, underscore_1.uniq)(result).length <= accepted;
        };
        diff = +1;
    }
    let maxIter = 100;
    do {
        result = uniqValues.map(currentRound);
        decimals += diff;
    } while (check() && maxIter-- > 0);
    if (maxIter < 10) {
        console.warn('maximum iteration reached', values, result, decimals);
    }
    if (diff < 0)
        decimals += 2;
    else
        decimals--;
    /* rounds to the current number of decimals */
    function currentRound(v) {
        return (0, round_js_1.default)(v, decimals);
    }
    return decimals;
}
exports.default = significantDimension;
