import { BrowserWindow } from './browserGlobals.js';
import { HttpReqOptions, SimpleFetchResponse } from './httpReqOptions.js';
declare global {
    interface Window extends BrowserWindow {
    }
}
/**
 * The response body is automatically parsed according
 * to the response content type.
 *
 * @exports httpReq
 * @kind function
 *
 * @param {string} path                - the url path that gets appended to baseUrl
 * @param {object} options.body        - raw body to be send with req
 * @param {object} options.payload     - raw JSON payload to be send with req (will overwrite options.body)
 * @param {boolean} options.raw        - disable parsing of response body, returns raw response
 * @param {string} options.baseUrl     - base for url, defaults to dw api domain
 * @param {string} options.disableCSRF - set to true to disable CSRF cookies
 * @param {*} options.*                - additional options for window.fetch or node-fetch
 *
 * @returns {Promise} promise of parsed response body or raw response
 *
 * @example
 *  import httpReq from '@datawrapper/shared/httpReq';
 *  let res = await httpReq('/v3/charts', {
 *      method: 'post',
 *      payload: {
 *          title: 'My new chart'
 *      }
 *  });
 *  import { post } from '@datawrapper/shared/httpReq';
 *  res = await post('/v3/charts', {
 *      payload: {
 *          title: 'My new chart'
 *      }
 *  });
 *  // send raw csv
 *  await httpReq.put(`/v3/charts/${chartId}/data`, {
 *       body: csvData,
 *       headers: {
 *           'Content-Type': 'text/csv'
 *       }
 *   });
 */
declare function httpReq(path: string, { fetch: inputFetch, baseUrl: inputBaseUrl, ...options }?: HttpReqOptions): Promise<any>;
declare namespace httpReq {
    export var get: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    export var patch: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    export var put: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    export var post: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    export var head: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    var _a: (path: string, options?: Omit<HttpReqOptions, "method">) => Promise<any>;
    export var HttpReqError: typeof import("./httpReq.js").HttpReqError;
    export { _a as delete };
}
/**
 * Like `httpReq` but with fixed http method GET
 * @see {@link httpReq}
 *
 * @exports httpReq.get
 * @kind function
 */
export declare const get: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
/**
 * Like `httpReq` but with fixed http method PATCH
 * @see {@link httpReq}
 *
 * @exports httpReq.patch
 * @kind function
 */
export declare const patch: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
/**
 * Like `httpReq` but with fixed http method PUT
 * @see {@link httpReq}
 *
 * @exports httpReq.put
 * @kind function
 */
export declare const put: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
/**
 * Like `httpReq` but with fixed http method POST
 * @see {@link httpReq}
 *
 * @exports httpReq.post
 * @kind function
 */
export declare const post: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
/**
 * Like `httpReq` but with fixed http method HEAD
 * @see {@link httpReq}
 *
 * @exports httpReq.head
 * @kind function
 */
export declare const head: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
/**
 * Like `httpReq` but with fixed http method DELETE
 * @see {@link httpReq}
 *
 * @exports httpReq.delete
 * @kind function
 */
declare const delete_: (path: string, options?: Omit<HttpReqOptions, 'method'>) => Promise<any>;
export { delete_ as delete };
type ErrorInfo = {
    type: string;
    details?: {
        path: string;
        type?: string;
    }[];
};
export declare class HttpReqError extends Error {
    status: number;
    statusText: string;
    message: string;
    response: SimpleFetchResponse;
    type?: string;
    translationKey?: string;
    details?: {
        path: string;
        type?: string;
        translationKey?: string;
    }[];
    constructor(res: SimpleFetchResponse, json?: ErrorInfo);
}
export default httpReq;
